@@ -1,5 +1,5 @@
 #include "../CommonCommandFlags.h"
-#include "../LabelTrack.h"
+#include "LabelTrack.h"
 #include "MixAndRender.h"
 
 #include "Prefs.h"
@@ -23,7 +23,7 @@
 #include "Viewport.h"
 #include "WaveTrack.h"
 #include "CommandContext.h"
-#include "../effects/EffectManager.h"
+#include "EffectManager.h"
 #include "../effects/EffectUI.h"
 #include "QualitySettings.h"
 #include "../tracks/playabletrack/wavetrack/ui/WaveTrackControls.h"
@@ -31,6 +31,7 @@
 #include "../widgets/ASlider.h"
 #include "AudacityMessageBox.h"
 #include "ProgressDialog.h"
+#include "DoEffect.h"
 
 #include <wx/combobox.h>
 
@@ -521,7 +522,7 @@ void SetTrackGain(AudacityProject &project, WaveTrack * wt, LWSlider * slider)
    wxASSERT(wt);
    float newValue = slider->Get();
 
-   wt->SetGain(newValue);
+   wt->SetVolume(newValue);
 
    ProjectHistory::Get( project )
       .PushState(XO("Adjusted gain"), XO("Gain"), UndoPush::CONSOLIDATE);
@@ -551,9 +552,8 @@ namespace {
 void OnStereoToMono(const CommandContext &context)
 {
    EffectUI::DoEffect(
-      EffectManager::Get().GetEffectByIdentifier(wxT("StereoToMono")),
-      context,
-      EffectManager::kConfigured);
+      PluginManager::Get().GetByCommandIdentifier(wxT("StereoToMono")),
+      context.project, EffectManager::kConfigured);
 }
 
 void OnMixAndRender(const CommandContext &context)
@@ -974,7 +974,7 @@ void OnTrackGain(const CommandContext &context)
    /// This will pop up the track gain dialog for specified track
    const auto track = TrackFocus::Get( project ).Get();
    if (track) track->TypeSwitch( [&](WaveTrack &wt) {
-      LWSlider *slider = WaveTrackControls::GainSlider( trackPanel, wt );
+      LWSlider *slider = WaveTrackControls::VolumeSlider( trackPanel, wt );
       if (slider->ShowDialog())
          SetTrackGain(project, &wt, slider);
    });
@@ -987,7 +987,7 @@ void OnTrackGainInc(const CommandContext &context)
 
    const auto track = TrackFocus::Get( project ).Get();
    if (track) track->TypeSwitch( [&](WaveTrack &wt) {
-      LWSlider *slider = WaveTrackControls::GainSlider( trackPanel, wt );
+      LWSlider *slider = WaveTrackControls::VolumeSlider( trackPanel, wt );
       slider->Increase(1);
       SetTrackGain(project, &wt, slider);
    });
@@ -1000,7 +1000,7 @@ void OnTrackGainDec(const CommandContext &context)
 
    const auto track = TrackFocus::Get( project ).Get();
    if (track) track->TypeSwitch( [&](WaveTrack &wt) {
-      LWSlider *slider = WaveTrackControls::GainSlider( trackPanel, wt );
+      LWSlider *slider = WaveTrackControls::VolumeSlider( trackPanel, wt );
       slider->Decrease(1);
       SetTrackGain(project, &wt, slider);
    });
@@ -1138,7 +1138,7 @@ auto TracksMenu()
             // hide it.
             [](AudacityProject&) -> std::unique_ptr<CommandItem> {
                const PluginID ID =
-                  EffectManager::Get().GetEffectByIdentifier(wxT("StereoToMono"));
+                  PluginManager::Get().GetByCommandIdentifier(wxT("StereoToMono"));
                const PluginDescriptor *plug = PluginManager::Get().GetPlugin(ID);
                if (plug && plug->IsEnabled())
                   return Command( wxT("Stereo to Mono"),
