@@ -10,15 +10,14 @@
 
 #include "SpectrumCache.h"
 
-#include <cmath>
+#include "SpectrogramSettings.h"
 #include "RealFFTf.h"
-#include "SampleTrackCache.h"
-#include "../../../../prefs/SpectrogramSettings.h"
+#include "Sequence.h"
 #include "Spectrum.h"
-#include "WaveClipUtilities.h"
+#include "WaveClipUIUtilities.h"
 #include "WaveTrack.h"
-
-class WaveTrack;
+#include "WideSampleSequence.h"
+#include <cmath>
 
 namespace {
 
@@ -73,19 +72,31 @@ void ComputeSpectrogramGainFactors
 
 }
 
-bool SpecCache::Matches
-   (int dirty_, double pixelsPerSecond,
-    const SpectrogramSettings &settings, double rate) const
+bool SpecCache::isWaveletAnalysis(const SpectrogramSettings& settings) const
 {
-   // Make a tolerant comparison of the pps values in this wise:
+    return settings.algorithm == SpectrogramSettings::algWavelet;
+}
+
+bool SpecCache::Matches(
+   int dirty_, double samplesPerPixel,
+   const SpectrogramSettings& settings) const
+{
+   // Make a tolerant comparison of the spp values in this wise:
    // accumulated difference of times over the number of pixels is less than
    // a sample period.
-   const double tstep = 1.0 / pixelsPerSecond;
-   const bool ppsMatch =
-      (fabs(tstep - 1.0 / pps) * len < (1.0 / rate));
+   const bool sppMatch = (fabs(samplesPerPixel - spp) * len < 1.0);
 
+   if (isWaveletAnalysis(settings))
+   {
+       return
+          sppMatch &&
+          dirty == dirty_ &&
+          frequencyGain == settings.frequencyGain &&
+          algorithm == settings.algorithm &&
+          windowSize == settings.WindowSize();
+   }
    return
-      ppsMatch &&
+      sppMatch &&
       dirty == dirty_ &&
       windowType == settings.windowType &&
       windowSize == settings.WindowSize() &&
@@ -94,14 +105,11 @@ bool SpecCache::Matches
       algorithm == settings.algorithm;
 }
 
-bool SpecCache::CalculateOneSpectrum
-   (const SpectrogramSettings &settings,
-    SampleTrackCache &waveTrackCache,
-    const int xx, const sampleCount numSamples,
-    double offset, double rate, double pixelsPerSecond,
-    int lowerBoundX, int upperBoundX,
-    const std::vector<float> &gainFactors,
-    float* __restrict scratch, float* __restrict out) const
+bool SpecCache::CalculateOneSpectrum(
+   const SpectrogramSettings& settings, const WaveChannelInterval& clip,
+   const int xx, double pixelsPerSecond, int lowerBoundX, int upperBoundX,
+   const std::vector<float>& gainFactors, float* __restrict scratch,
+   float* __restrict out) const
 {
    bool result = false;
    const bool reassignment =
@@ -110,18 +118,18 @@ bool SpecCache::CalculateOneSpectrum
 
    sampleCount from;
 
+   const auto numSamples = clip.GetSequence().GetNumSamples();
+   const auto sampleRate = clip.GetRate();
+   const auto stretchRatio = clip.GetStretchRatio();
+   const auto samplesPerPixel = sampleRate / pixelsPerSecond / stretchRatio;
    // xx may be for a column that is out of the visible bounds, but only
    // when we are calculating reassignment contributions that may cross into
    // the visible area.
 
    if (xx < 0)
-      from = sampleCount(
-         where[0].as_double() + xx * (rate / pixelsPerSecond)
-      );
+      from = sampleCount(where[0].as_double() + xx * samplesPerPixel);
    else if (xx > (int)len)
-      from = sampleCount(
-         where[len].as_double() + (xx - len) * (rate / pixelsPerSecond)
-      );
+      from = sampleCount(where[len].as_double() + (xx - len) * samplesPerPixel);
    else
       from = where[xx];
 
@@ -144,7 +152,8 @@ bool SpecCache::CalculateOneSpectrum
 
       // We can avoid copying memory when ComputeSpectrum is used below
       bool copy = !autocorrelation || (padding > 0) || reassignment;
-      float *useBuffer = 0;
+      std::vector<float> floats;
+      float* useBuffer = 0;
       float *adj = scratch + padding;
 
       {
@@ -172,15 +181,13 @@ bool SpecCache::CalculateOneSpectrum
          }
 
          if (myLen > 0) {
-            useBuffer = (float*)(waveTrackCache.GetFloats(
-               sampleCount(
-                  floor(0.5 + from.as_double() + offset * rate)
-               ),
-               myLen,
-               // Don't throw in this drawing operation
-               false)
-            );
-
+            constexpr auto iChannel = 0u;
+            constexpr auto mayThrow = false; // Don't throw just for display
+            mSampleCacheHolder.emplace(
+               clip.GetSampleView(from, myLen, mayThrow));
+            floats.resize(myLen);
+            mSampleCacheHolder->Copy(floats.data(), myLen);
+            useBuffer = floats.data();
             if (copy) {
                if (useBuffer)
                   memcpy(adj, useBuffer, myLen * sizeof(float));
@@ -198,8 +205,8 @@ bool SpecCache::CalculateOneSpectrum
          wxASSERT(xx >= 0);
          float *const results = &out[nBins * xx];
          // This function does not mutate useBuffer
-         ComputeSpectrum(useBuffer, windowSizeSetting, windowSizeSetting,
-            rate, results,
+         ComputeSpectrum(
+            useBuffer, windowSizeSetting, windowSizeSetting, results,
             autocorrelation, settings.windowType);
       }
       else if (reassignment) {
@@ -276,7 +283,11 @@ bool SpecCache::CalculateOneSpectrum
                      (numRe * denomRe + numIm * denomIm) / power;
                }
 
-               int correctedX = (floor(0.5 + xx + timeCorrection * pixelsPerSecond / rate));
+               // PRL: timeCorrection is scaled to the clip's raw sample rate,
+               // without the stretching ratio correction for real time. We want
+               // to find the correct X coordinate for that.
+               int correctedX = (floor(
+                  0.5 + xx + timeCorrection * pixelsPerSecond / sampleRate));
                if (correctedX >= lowerBoundX && correctedX < upperBoundX)
                {
                   result = true;
@@ -318,8 +329,132 @@ bool SpecCache::CalculateOneSpectrum
    return result;
 }
 
-void SpecCache::Grow(size_t len_, SpectrogramSettings& settings,
-                       double pixelsPerSecond, double start_)
+bool SpecCache::CalculateOneWaveletSpectrum(
+   const SpectrogramSettings& settings, const WaveChannelInterval& clip,
+   const int xx, double pixelsPerSecond, int lowerBoundX, int upperBoundX,
+   const std::vector<float>& gainFactors,
+   float* __restrict out) const
+{
+   bool result = false;
+   sampleCount from;
+
+   const auto numSamples = clip.GetSequence().GetNumSamples();
+   const auto sampleRate = clip.GetRate();
+   const auto stretchRatio = clip.GetStretchRatio();
+   const auto samplesPerPixel = sampleRate / pixelsPerSecond / stretchRatio;
+   const size_t nBins = settings.NBins();
+   float *const results = &out[nBins * xx];
+
+   // not reassignment, xx is surely within bounds.
+   wxASSERT(xx >= 0);
+   if (xx > (int)len)
+      from = sampleCount(where[len].as_double() + (xx - len) * samplesPerPixel);
+   else
+      from = where[xx];
+
+   if (from < 0 || from >= numSamples) {
+      if (xx >= 0 && xx < (int)len) {
+         // Pixel column is out of bounds of the clip!  Should not happen.
+         std::fill(results, results + nBins, 0.0f);
+      }
+   }
+   else
+   {
+      // Iterate frequencies in turn
+      // Note that as opposed to Fourier Transform, using wavelets implies a window length that varies with frequency
+      // We extract samples corresponding to max wavelet length (the first)
+    
+      // Locate longest wavelet (first one)
+      const auto longestWaveletLen = settings.waveletMaxLength;
+      wxASSERT(longestWaveletLen);
+       
+      auto myLen = 2 * longestWaveletLen - 1;
+      std::vector<float> scratch(myLen);
+
+      // Take a window of the track centered at this sample.
+      from -= longestWaveletLen;
+      float * adj = &scratch[0];
+      if (from < 0) {
+         // Near the start of the clip, pad left with zeroes as needed.
+         for (auto ii = from; ii < 0; ++ii)
+             *adj++ = 0;
+         myLen += from.as_long_long(); // add a negative
+         from = 0;
+      }
+      if (from + myLen >= numSamples) {
+         // Near the end of the clip, pad right with zeroes as needed.
+         // newlen is bounded by myLen:
+         auto newlen = ( numSamples - from ).as_size_t();
+         for (decltype(myLen) ii = newlen; ii < myLen; ++ii)
+             adj[ii] = 0;
+         myLen = newlen;
+      }
+      std::vector<float> floats;
+      if (myLen > 0) {
+         constexpr auto iChannel = 0u;
+         constexpr auto mayThrow = false; // Don't throw just for display
+         mSampleCacheHolder.emplace(
+             clip.GetSampleView(from, myLen, mayThrow));
+         floats.resize(myLen);
+         mSampleCacheHolder->Copy(floats.data(), myLen);
+         float *useBuffer = floats.data();
+         memcpy(adj, useBuffer, myLen * sizeof(float));
+      }
+
+      // Done preparing samples. Now onward with calculations
+      const float * middleSample = &scratch[longestWaveletLen - 1];
+      for (size_t iBin = 0; iBin < nBins; iBin++)
+      {
+          if (settings.waveletSizes[iBin] == 0)
+          {
+             results[iBin] = -160.0;
+             continue;
+          }
+         
+          const float *right = middleSample;
+          const float *left = middleSample;
+          // Do the inner product
+          const float * wRe = &settings.waveletsRe[iBin][0];
+          const float * wIm = &settings.waveletsIm[iBin][0];
+          double sumRe = *middleSample * *wRe;
+          double sumIm = 0.0;
+          wRe++, wIm++, right++, left--;
+          const auto halfLen = settings.waveletSizes[iBin];
+          for (auto ii = 1; ii < halfLen; ii++)
+          {
+              sumRe +=  (*right + *left) * *wRe;
+              sumIm +=  (*left - *right) * *wIm;
+              wRe++, wIm++, right++, left--;
+          }
+
+          // Calculate magnitude
+          float power = sumRe * sumRe + sumIm * sumIm;
+          
+          
+          // Do the log thing
+          if (power <= 0)
+             power = -160.0;
+          else
+             power = 10.0*log10f(power);
+
+          results[iBin] = power;
+      }
+
+
+       if (!gainFactors.empty()) {
+          // Apply a frequency-dependent gain factor
+          for (size_t ii = 0; ii < nBins; ++ii)
+             results[ii] += gainFactors[ii];
+       }
+       result = true;
+   }
+
+   return result;
+}
+
+void SpecCache::Grow(
+   size_t len_, SpectrogramSettings& settings, double samplesPerPixel,
+   double start_)
 {
    settings.CacheWindows();
 
@@ -333,7 +468,7 @@ void SpecCache::Grow(size_t len_, SpectrogramSettings& settings,
 
    len = len_;
    algorithm = settings.algorithm;
-   pps = pixelsPerSecond;
+   spp = samplesPerPixel;
    start = start_;
    windowType = settings.windowType;
    windowSize = settings.WindowSize();
@@ -341,12 +476,11 @@ void SpecCache::Grow(size_t len_, SpectrogramSettings& settings,
    frequencyGain = settings.frequencyGain;
 }
 
-void SpecCache::Populate
-   (const SpectrogramSettings &settings, SampleTrackCache &waveTrackCache,
-    int copyBegin, int copyEnd, size_t numPixels,
-    sampleCount numSamples,
-    double offset, double rate, double pixelsPerSecond)
+void SpecCache::Populate(
+   const SpectrogramSettings& settings, const WaveChannelInterval& clip,
+   int copyBegin, int copyEnd, size_t numPixels, double pixelsPerSecond)
 {
+   const auto sampleRate = clip.GetRate();
    const int &frequencyGainSetting = settings.frequencyGain;
    const size_t windowSizeSetting = settings.WindowSize();
    const bool autocorrelation =
@@ -366,7 +500,8 @@ void SpecCache::Populate
 
    std::vector<float> gainFactors;
    if (!autocorrelation)
-      ComputeSpectrogramGainFactors(fftLen, rate, frequencyGainSetting, gainFactors);
+      ComputeSpectrogramGainFactors(
+         fftLen, sampleRate, frequencyGainSetting, gainFactors);
 
    // Loop over the ranges before and after the copied portion and compute anew.
    // One of the ranges may be empty.
@@ -374,6 +509,8 @@ void SpecCache::Populate
       const int lowerBoundX = jj == 0 ? 0 : copyEnd;
       const int upperBoundX = jj == 0 ? copyBegin : numPixels;
 
+// todo(mhodgkinson): I don't find an option to define _OPENMP anywhere. Is this
+// still of interest?
 #ifdef _OPENMP
       // Storage for mutable per-thread data.
       // private clause ensures one copy per thread
@@ -395,19 +532,24 @@ void SpecCache::Populate
 #endif
       for (auto xx = lowerBoundX; xx < upperBoundX; ++xx)
       {
+         if (isWaveletAnalysis(settings))
+         {
+             CalculateOneWaveletSpectrum(settings, clip, xx, pixelsPerSecond, lowerBoundX, upperBoundX,
+                                         gainFactors, &freq[0]);
+         }
+         else
+         {
 #ifdef _OPENMP
-         tls.init(waveTrackCache, scratchSize);
-         SampleTrackCache& cache = *tls.cache;
-         float* buffer = &tls.scratch[0];
+             tls.init(waveTrackCache, scratchSize);
+             SampleTrackCache& cache = *tls.cache;
+             float* buffer = &tls.scratch[0];
 #else
-         SampleTrackCache& cache = waveTrackCache;
-         float* buffer = &scratch[0];
+             float* buffer = &scratch[0];
 #endif
-         CalculateOneSpectrum(
-            settings, cache, xx, numSamples,
-            offset, rate, pixelsPerSecond,
-            lowerBoundX, upperBoundX,
-            gainFactors, buffer, &freq[0]);
+             CalculateOneSpectrum(
+                         settings, clip, xx, pixelsPerSecond, lowerBoundX, upperBoundX,
+                         gainFactors, buffer, &freq[0]);
+         }
       }
 
       if (reassignment) {
@@ -415,16 +557,14 @@ void SpecCache::Populate
          // time reassignments.
          // I'm not sure what's a good stopping criterion?
          auto xx = lowerBoundX;
-         const double pixelsPerSample = pixelsPerSecond / rate;
+         const double pixelsPerSample =
+            pixelsPerSecond * clip.GetStretchRatio() / sampleRate;
          const int limit = std::min((int)(0.5 + fftLen * pixelsPerSample), 100);
          for (int ii = 0; ii < limit; ++ii)
          {
-            const bool result =
-               CalculateOneSpectrum(
-                  settings, waveTrackCache, --xx, numSamples,
-                  offset, rate, pixelsPerSecond,
-                  lowerBoundX, upperBoundX,
-                  gainFactors, &scratch[0], &freq[0]);
+            const bool result = CalculateOneSpectrum(
+               settings, clip, --xx, pixelsPerSecond, lowerBoundX, upperBoundX,
+               gainFactors, &scratch[0], &freq[0]);
             if (!result)
                break;
          }
@@ -432,12 +572,9 @@ void SpecCache::Populate
          xx = upperBoundX;
          for (int ii = 0; ii < limit; ++ii)
          {
-            const bool result =
-               CalculateOneSpectrum(
-                  settings, waveTrackCache, xx++, numSamples,
-                  offset, rate, pixelsPerSecond,
-                  lowerBoundX, upperBoundX,
-                  gainFactors, &scratch[0], &freq[0]);
+            const bool result = CalculateOneSpectrum(
+               settings, clip, xx++, pixelsPerSecond, lowerBoundX, upperBoundX,
+               gainFactors, &scratch[0], &freq[0]);
             if (!result)
                break;
          }
@@ -466,33 +603,28 @@ void SpecCache::Populate
    }
 }
 
-bool WaveClipSpectrumCache::GetSpectrogram(const WaveClip &clip,
-   SampleTrackCache &waveTrackCache,
-   const float *& spectrogram,
-   const sampleCount *& where,
-   size_t numPixels,
-   double t0, double pixelsPerSecond)
-{
-   t0 += clip.GetTrimLeft();
+bool WaveClipSpectrumCache::GetSpectrogram(
+   const WaveChannelInterval &clip,
+   const float*& spectrogram, SpectrogramSettings& settings,
+   const sampleCount*& where, size_t numPixels, double t0,
+   double pixelsPerSecond)
 
-   const auto track =
-      static_cast<const WaveTrack*>(waveTrackCache.GetTrack().get());
-   auto &settings = SpectrogramSettings::Get(*track);
-   const auto rate = clip.GetRate();
+{
+   auto &mSpecCache = mSpecCaches[clip.GetChannelIndex()];
+
+   const auto sampleRate = clip.GetRate();
+   const auto stretchRatio = clip.GetStretchRatio();
+   const auto samplesPerPixel = sampleRate / pixelsPerSecond / stretchRatio;
 
    //Trim offset comparison failure forces spectrogram cache rebuild
    //and skip copying "unchanged" data after clip border was trimmed.
-   bool match =
-      mSpecCache &&
-      mSpecCache->leftTrim == clip.GetTrimLeft() &&
-      mSpecCache->rightTrim == clip.GetTrimRight() &&
-      mSpecCache->len > 0 &&
-      mSpecCache->Matches
-      (mDirty, pixelsPerSecond, settings, rate);
+   bool match = mSpecCache && mSpecCache->leftTrim == clip.GetTrimLeft() &&
+                mSpecCache->rightTrim == clip.GetTrimRight() &&
+                mSpecCache->len > 0 &&
+                mSpecCache->Matches(mDirty, samplesPerPixel, settings);
 
-   if (match &&
-       mSpecCache->start == t0 &&
-       mSpecCache->len >= numPixels) {
+   if (match && mSpecCache->start == t0 && mSpecCache->len >= numPixels)
+   {
       spectrogram = &mSpecCache->freq[0];
       where = &mSpecCache->where[0];
 
@@ -516,28 +648,22 @@ bool WaveClipSpectrumCache::GetSpectrogram(const WaveClip &clip,
       mSpecCache = std::make_unique<SpecCache>();
    }
 
-   const double tstep = 1.0 / pixelsPerSecond;
-   const double samplesPerPixel = rate * tstep;
-
    int oldX0 = 0;
    double correction = 0.0;
 
    int copyBegin = 0, copyEnd = 0;
    if (match) {
-      findCorrection(mSpecCache->where, mSpecCache->len, numPixels,
-         t0, rate, samplesPerPixel,
-         oldX0, correction);
+      WaveClipUIUtilities::findCorrection(
+         mSpecCache->where, mSpecCache->len, numPixels, t0, sampleRate,
+         stretchRatio, samplesPerPixel, oldX0, correction);
       // Remember our first pixel maps to oldX0 in the old cache,
       // possibly out of bounds.
       // For what range of pixels can data be copied?
-      copyBegin = std::min((int)numPixels, std::max(0, -oldX0));
-      copyEnd = std::min((int)numPixels, std::max(0,
-         (int)mSpecCache->len - oldX0
-      ));
+      int oldLen = mSpecCache->len;
+      copyBegin = std::min(oldLen, std::max(0, -oldX0));
+      copyEnd = std::min(oldLen, std::max(0, oldLen - oldX0));
    }
 
-   // Resize the cache, keep the contents unchanged.
-   mSpecCache->Grow(numPixels, settings, pixelsPerSecond, t0);
    mSpecCache->leftTrim = clip.GetTrimLeft();
    mSpecCache->rightTrim = clip.GetTrimRight();
    auto nBins = settings.NBins();
@@ -553,6 +679,9 @@ bool WaveClipSpectrumCache::GetSpectrogram(const WaveClip &clip,
                nBins * (copyEnd - copyBegin) * sizeof(float));
    }
 
+   // Resize the cache, keep the contents unchanged.
+   mSpecCache->Grow(numPixels, settings, samplesPerPixel, t0);
+
    // Reassignment accumulates, so it needs a zeroed buffer
    if (settings.algorithm == SpectrogramSettings::algReassignment)
    {
@@ -573,14 +702,12 @@ bool WaveClipSpectrumCache::GetSpectrogram(const WaveClip &clip,
 
    // purposely offset the display 1/2 sample to the left (as compared
    // to waveform display) to properly center response of the FFT
-   fillWhere(mSpecCache->where, numPixels, 0.5, correction,
-      t0, rate, samplesPerPixel);
-
-   mSpecCache->Populate
-      (settings, waveTrackCache, copyBegin, copyEnd, numPixels,
-       clip.GetSequenceSamplesCount(),
-       clip.GetSequenceStartTime(), rate, pixelsPerSecond);
+   constexpr auto addBias = true;
+   WaveClipUIUtilities::fillWhere(
+      mSpecCache->where, numPixels, addBias, correction, t0, sampleRate,
+      stretchRatio, samplesPerPixel);
 
+   mSpecCache->Populate(settings, clip, copyBegin, copyEnd, numPixels, pixelsPerSecond);
    mSpecCache->dirty = mDirty;
    spectrogram = &mSpecCache->freq[0];
    where = &mSpecCache->where[0];
@@ -588,27 +715,36 @@ bool WaveClipSpectrumCache::GetSpectrogram(const WaveClip &clip,
    return true;
 }
 
-WaveClipSpectrumCache::WaveClipSpectrumCache()
-: mSpecCache{ std::make_unique<SpecCache>() }
-, mSpecPxCache{ std::make_unique<SpecPxCache>(1) }
+WaveClipSpectrumCache::WaveClipSpectrumCache(size_t nChannels)
+   : mSpecCaches(nChannels)
+   , mSpecPxCaches(nChannels)
 {
+   for (auto &pCache : mSpecCaches)
+      pCache = std::make_unique<SpecCache>();
 }
 
 WaveClipSpectrumCache::~WaveClipSpectrumCache()
 {
 }
 
-static WaveClip::Caches::RegisteredFactory sKeyS{ []( WaveClip& ){
-   return std::make_unique< WaveClipSpectrumCache >();
-} };
-
-WaveClipSpectrumCache &WaveClipSpectrumCache::Get( const WaveClip &clip )
+std::unique_ptr<WaveClipListener> WaveClipSpectrumCache::Clone() const
 {
-   return const_cast< WaveClip& >( clip ) // Consider it mutable data
-      .Caches::Get< WaveClipSpectrumCache >( sKeyS );
+   // Don't need to copy contents
+   return std::make_unique<WaveClipSpectrumCache>(mSpecCaches.size());
 }
 
-void WaveClipSpectrumCache::MarkChanged()
+static WaveClip::Attachments::RegisteredFactory sKeyS{ [](WaveClip &clip){
+   return std::make_unique<WaveClipSpectrumCache>(clip.NChannels());
+} };
+
+WaveClipSpectrumCache &
+WaveClipSpectrumCache::Get(const WaveChannelInterval &clip)
+{
+   return const_cast<WaveClip&>(clip.GetClip()) // Consider it mutable data
+      .Attachments::Get< WaveClipSpectrumCache >( sKeyS );
+}
+
+void WaveClipSpectrumCache::MarkChanged() noexcept
 {
    ++mDirty;
 }
@@ -616,5 +752,30 @@ void WaveClipSpectrumCache::MarkChanged()
 void WaveClipSpectrumCache::Invalidate()
 {
    // Invalidate the spectrum display cache
-   mSpecCache = std::make_unique<SpecCache>();
+   for (auto &pCache : mSpecCaches)
+      pCache = std::make_unique<SpecCache>();
+}
+
+void WaveClipSpectrumCache::MakeStereo(WaveClipListener &&other, bool)
+{
+   auto pOther = dynamic_cast<WaveClipSpectrumCache *>(&other);
+   assert(pOther); // precondition
+   mSpecCaches.push_back(move(pOther->mSpecCaches[0]));
+   mSpecPxCaches.push_back(move(pOther->mSpecPxCaches[0]));
+}
+
+void WaveClipSpectrumCache::SwapChannels()
+{
+   mSpecCaches.resize(2);
+   std::swap(mSpecCaches[0], mSpecCaches[1]);
+   mSpecPxCaches.resize(2);
+   std::swap(mSpecPxCaches[0], mSpecPxCaches[1]);
+}
+
+void WaveClipSpectrumCache::Erase(size_t index)
+{
+   if (index < mSpecCaches.size())
+      mSpecCaches.erase(mSpecCaches.begin() + index);
+   if (index < mSpecPxCaches.size())
+      mSpecPxCaches.erase(mSpecPxCaches.begin() + index);
 }
